@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 1986, 1988, 1989, 1991-2019, 2021-2025,
+ * Copyright (C) 1986, 1988, 1989, 1991-2019, 2021-2026,
  * the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
@@ -504,12 +504,21 @@ genflags2str(int flagval, const struct flagtab *tab)
 
 /* posix_compare --- compare strings using strcoll */
 
+/*
+ * 1/2026:
+ * On systems except MingGW (even Cygwin), using strcoll() to do the
+ * work looks to be enough.  So that's the default. On MinGW, we do
+ * different tests based on gawk_mb_cur_max.
+ */
+
 static int
 posix_compare(NODE *s1, NODE *s2)
 {
-	int ret;
+	int ret = 0;
 
+#ifdef __MINGW32__
 	if (gawk_mb_cur_max == 1) {
+#endif
 		char save1, save2;
 		const char *p1, *p2;
 
@@ -546,10 +555,10 @@ posix_compare(NODE *s1, NODE *s2)
 
 		s1->stptr[s1->stlen] = save1;
 		s2->stptr[s2->stlen] = save2;
-	}
-	else {
+#ifdef __MINGW32__
+	} else {
 		/* Similar logic, using wide characters */
-		const wchar_t *p1, *p2;
+		const char32_t *p1, *p2;
 
 		(void) force_wstring(s1);
 		(void) force_wstring(s2);
@@ -560,11 +569,11 @@ posix_compare(NODE *s1, NODE *s2)
 		for (;;) {
 			size_t len;
 
-			ret = wcscoll(p1, p2);
+			ret = c32scoll(p1, p2);
 			if (ret != 0)
 				break;
 
-			len = wcslen(p1);
+			len = c32slen(p1);
 			p1 += len + 1;
 			p2 += len + 1;
 
@@ -579,6 +588,7 @@ posix_compare(NODE *s1, NODE *s2)
 			}
 		}
 	}
+#endif /* __MINGW32__ */
 
 	return ret;
 }
@@ -1013,6 +1023,9 @@ update_ERRNO_int(int errcode)
 {
 	const char *cp;
 
+	if (do_traditional)
+		return;
+
 	update_PROCINFO_num("errno", errcode);
 	if (errcode) {
 		cp = strerror(errcode);
@@ -1028,6 +1041,9 @@ update_ERRNO_int(int errcode)
 void
 update_ERRNO_string(const char *string)
 {
+	if (do_traditional)
+		return;
+
 	update_PROCINFO_num("errno", 0);
 	unref(ERRNO_node->var_value);
 	size_t len = strlen(string);
@@ -1054,6 +1070,9 @@ update_ERRNO_string(const char *string)
 void
 unset_ERRNO(void)
 {
+	if (do_traditional)
+		return;
+
 	update_PROCINFO_num("errno", 0);
 	unref(ERRNO_node->var_value);
 	ERRNO_node->var_value = dupnode(Nnull_string);
