@@ -3,7 +3,8 @@
  */
 
 /*
- * Copyright (C) 2012-2019, 2021, 2022, the Free Software Foundation, Inc.
+ * Copyright (C) 2012-2019, 2021-2025,
+ * the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
  * AWK Programming Language.
@@ -72,16 +73,17 @@ api_get_argument(awk_ext_id_t id, size_t count,
 	if (arg->type == Node_var_new || arg->type == Node_elem_new) {
 		if (wanted == AWK_UNDEFINED)
 			return awk_true;
-		else if (wanted == AWK_ARRAY) {
+		else if (wanted == AWK_ARRAY)
 			goto array;
-		} else {
+		else
 			goto scalar;
-		}
 	}
 
 	/* at this point, we have real type */
 	if (arg->type == Node_var_array || arg->type == Node_array_ref) {
-		if (wanted != AWK_ARRAY && wanted != AWK_UNDEFINED)
+		if (wanted == AWK_UNDEFINED)
+			return awk_true;
+		else if (wanted != AWK_ARRAY)
 			return awk_false;
 		goto array;
 	} else
@@ -431,7 +433,7 @@ api_awk_atexit(awk_ext_id_t id,
 		return;
 
 	/* allocate memory */
-	emalloc(p, struct ext_exit_handler *, sizeof(struct ext_exit_handler), "api_awk_atexit");
+	emalloc(p, struct ext_exit_handler *, sizeof(struct ext_exit_handler));
 
 	/* fill it in */
 	p->funcp = funcp;
@@ -479,9 +481,9 @@ assign_string(NODE *node, awk_value_t *val, awk_valtype_t val_type)
 				scopy.size = 8;	/* initial size */
 			else
 				scopy.size *= 2;
-			erealloc(scopy.strings, char **, scopy.size * sizeof(char *), "assign_string");
+			erealloc(scopy.strings, char **, scopy.size * sizeof(char *));
 		}
-		emalloc(s, char *, node->stlen + 1, "assign_string");
+		emalloc(s, char *, node->stlen + 1);
 		memcpy(s, node->stptr, node->stlen);
 		s[node->stlen] = '\0';
 		val->str_value.str = scopy.strings[scopy.i++] = s;
@@ -624,11 +626,9 @@ node_to_awk_value(NODE *node, awk_value_t *val, awk_valtype_t wanted)
 				val->val_type = AWK_REGEX;
 				break;
 			case NUMBER|STRING:
-				if (node == Nnull_string) {
-					val->val_type = AWK_UNDEFINED;
-					break;
-				}
-				/* fall through */
+				// this can come from a Node_elem_new, as well as Nnull_string
+				val->val_type = AWK_UNDEFINED;
+				break;
 			default:
 				warning(_("node_to_awk_value detected invalid flags combination `%s'; please file a bug report"), flags2str(node->flags));
 				val->val_type = AWK_UNDEFINED;
@@ -661,11 +661,9 @@ node_to_awk_value(NODE *node, awk_value_t *val, awk_valtype_t wanted)
 				ret = awk_true;
 				break;
 			case NUMBER|STRING:
-				if (node == Nnull_string) {
-					val->val_type = AWK_UNDEFINED;
-					break;
-				}
-				/* fall through */
+				// this can come from a Node_elem_new, as well as Nnull_string
+				val->val_type = AWK_UNDEFINED;
+				break;
 			default:
 				warning(_("node_to_awk_value detected invalid flags combination `%s'; please file a bug report"), flags2str(node->flags));
 				val->val_type = AWK_UNDEFINED;
@@ -691,11 +689,9 @@ node_to_awk_value(NODE *node, awk_value_t *val, awk_valtype_t wanted)
 				val->val_type = AWK_REGEX;
 				break;
 			case NUMBER|STRING:
-				if (node == Nnull_string) {
-					val->val_type = AWK_UNDEFINED;
-					break;
-				}
-				/* fall through */
+				// this can come from a Node_elem_new, as well as Nnull_string
+				val->val_type = AWK_UNDEFINED;
+				break;
 			default:
 				warning(_("node_to_awk_value detected invalid flags combination `%s'; please file a bug report"), flags2str(node->flags));
 				val->val_type = AWK_UNDEFINED;
@@ -727,12 +723,10 @@ node_to_awk_value(NODE *node, awk_value_t *val, awk_valtype_t wanted)
 				ret = awk_true;
 				break;
 			case NUMBER|STRING:
-				if (node == Nnull_string) {
-					val->val_type = AWK_UNDEFINED;
-					ret = awk_true;
-					break;
-				}
-				/* fall through */
+				// this can come from a Node_elem_new, as well as Nnull_string
+				val->val_type = AWK_UNDEFINED;
+				ret = awk_true;
+				break;
 			default:
 				warning(_("node_to_awk_value detected invalid flags combination `%s'; please file a bug report"), flags2str(node->flags));
 				val->val_type = AWK_UNDEFINED;
@@ -916,9 +910,12 @@ api_sym_update(awk_ext_id_t id,
 		unref(node->var_value);
 		node->var_value = awk_value_to_node(value);
 		if ((node->type == Node_var_new || node->type == Node_elem_new)
-		    && value->val_type != AWK_UNDEFINED)
+		    && value->val_type != AWK_UNDEFINED) {
+			if (node->type == Node_elem_new) {
+				elem_new_reset(node);
+			}
 			node->type = Node_var;
-
+		}
 		return awk_true;
 	}
 
@@ -1107,8 +1104,9 @@ api_set_array_element(awk_ext_id_t id, awk_array_t a_cookie,
 	elem = awk_value_to_node(value);
 	if (elem->type == Node_var_array) {
 		elem->parent_array = array;
-		elem->vname = estrdup(index->str_value.str,
-					index->str_value.len);
+		// array indices are strings, ensure that this is the case
+		(void) force_string(tmp);
+		elem->vname = estrdup(tmp->stptr, tmp->stlen);
 	}
 	assoc_set(array, tmp, elem);
 
@@ -1254,8 +1252,7 @@ api_flatten_array_typed(awk_ext_id_t id,
 	alloc_size = sizeof(awk_flat_array_t) +
 			(array->table_size - 1) * sizeof(awk_element_t);
 
-	ezalloc(*data, awk_flat_array_t *, alloc_size,
-			"api_flatten_array_typed");
+	ezalloc(*data, awk_flat_array_t *, alloc_size);
 
 	list = assoc_list(array, "@unsorted", ASORTI);
 
@@ -1369,7 +1366,7 @@ api_get_mpfr(awk_ext_id_t id)
 {
 #ifdef HAVE_MPFR
 	mpfr_ptr p;
-	emalloc(p, mpfr_ptr, sizeof(mpfr_t), "api_get_mpfr");
+	emalloc(p, mpfr_ptr, sizeof(mpfr_t));
 	mpfr_init(p);
 	return p;
 #else
@@ -1385,7 +1382,7 @@ api_get_mpz(awk_ext_id_t id)
 {
 #ifdef HAVE_MPFR
 	mpz_ptr p;
-	emalloc(p, mpz_ptr, sizeof (mpz_t), "api_get_mpz");
+	emalloc(p, mpz_ptr, sizeof (mpz_t));
 
 	mpz_init(p);
 	return p;
@@ -1511,7 +1508,7 @@ api_register_ext_version(awk_ext_id_t id, const char *version)
 
 	(void) id;
 
-	emalloc(info, struct version_info *, sizeof(struct version_info), "register_ext_version");
+	emalloc(info, struct version_info *, sizeof(struct version_info));
 	info->version = version;
 	info->next = vi_head;
 	vi_head = info;
@@ -1600,12 +1597,13 @@ void
 init_ext_api()
 {
 	/* force values to 1 / 0 */
-	api_impl.do_flags[0] = (do_lint ? 1 : 0);
-	api_impl.do_flags[1] = (do_traditional ? 1 : 0);
-	api_impl.do_flags[2] = (do_profile ? 1 : 0);
-	api_impl.do_flags[3] = (do_sandbox ? 1 : 0);
-	api_impl.do_flags[4] = (do_debug ? 1 : 0);
-	api_impl.do_flags[5] = (do_mpfr ? 1 : 0);
+	api_impl.do_flags[gawk_do_lint] = (do_lint ? 1 : 0);
+	api_impl.do_flags[gawk_do_traditional] = (do_traditional ? 1 : 0);
+	api_impl.do_flags[gawk_do_profile] = (do_profile ? 1 : 0);
+	api_impl.do_flags[gawk_do_sandbox] = (do_sandbox ? 1 : 0);
+	api_impl.do_flags[gawk_do_debug] = (do_debug ? 1 : 0);
+	api_impl.do_flags[gawk_do_mpfr] = (do_mpfr ? 1 : 0);
+	api_impl.do_flags[gawk_do_csv] = (do_csv ? 1 : 0);
 }
 
 /* update_ext_api --- update the variables in the API that can change */
@@ -1670,7 +1668,7 @@ ns_lookup(const char *name_space, const char *name, char **fullname)
 
 	size_t len = strlen(name_space) + 2 + strlen(name) + 1;
 	char *buf;
-	emalloc(buf, char *, len, "ns_lookup");
+	emalloc(buf, char *, len);
 	sprintf(buf, "%s::%s", name_space, name);
 
 	NODE *f = lookup(buf);
