@@ -3,7 +3,8 @@
  */
 
 /*
- * Copyright (C) 1986, 1988, 1989, 1991-2004, 2010, 2011, 2017,
+ * Copyright (C) 1986, 1988, 1989, 1991-2004, 2010, 2011, 2017, 2022, 2023,
+ * 2026
  * the Free Software Foundation, Inc.
  *
  * This file is part of GAWK, the GNU implementation of the
@@ -30,14 +31,11 @@
 #include <fcntl.h>
 #endif
 
-/* some old compilers don't grok #elif. sigh */
-
-#if defined(__EMX__) || defined(__DJGPP__) || defined(__MINGW32__)
+#if defined(__MINGW32__)
+#include "pc/mbc32.c"
 #include "pc/gawkmisc.pc"
-#else /* not __DJGPP__, not __MINGW32__ */
-#if defined(VMS)
+#elif defined(VMS)
 #include "vms/gawkmisc.vms"
-#else /* not VMS */
+#else /* posix */
 #include "posix/gawkmisc.c"
-#endif /* not VMS */
-#endif /* not __DJGPP__, not __MINGW32__ */
+#endif
